'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _constants = require('./constants');

var _watcher = require('./watcher');

var _watcher2 = _interopRequireDefault(_watcher);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var getViewportHeight = function getViewportHeight(element) {
  var height = void 0;
  if (_constants.isOnServer) {
    height = 0;
  }

  if (element === document.body) {
    height = window.innerHeight || document.documentElement.clientHeight;
  } else {
    height = element.clientHeight;
  }
  return height;
};

var getContentHeight = function getContentHeight(element) {
  var height = void 0;
  if (_constants.isOnServer) {
    height = 0;
  }

  if (element === document.body) {
    height = Math.max(document.body.scrollHeight, document.documentElement.scrollHeight, document.body.offsetHeight, document.documentElement.offsetHeight, document.documentElement.clientHeight);
  } else {
    height = element.scrollHeight;
  }
  return height;
};

var scrollTop = function scrollTop(element) {
  var height = void 0;
  if (_constants.isOnServer) {
    height = 0;
  }
  if (element === document.body) {
    height = window.pageYOffset || document.documentElement && document.documentElement.scrollTop || document.body.scrollTop;
  } else {
    height = element.scrollTop;
  }
  return height;
};

var ScrollMonitorContainer = function () {
  function ScrollMonitorContainer(item, parentWatcher) {
    var _this = this;

    _classCallCheck(this, ScrollMonitorContainer);

    var self = this;
    this.item = item;
    this.watchers = [];
    this.viewportTop = null;
    this.viewportBottom = null;
    this.documentHeight = getContentHeight(item);
    this.viewportHeight = getViewportHeight(item);

    this.DOMListener = function (event) {
      _this.setStateFromDOM(event);
      _this.updateAndTriggerWatchers(event);
    };

    this.eventTypes = _constants.eventTypes;

    if (parentWatcher) {
      this.containerWatcher = parentWatcher.create(item);
    }

    var previousDocumentHeight = void 0;
    var calculateViewportI = void 0;

    var calculateViewport = function calculateViewport() {
      self.viewportTop = scrollTop(item);
      self.viewportBottom = self.viewportTop + self.viewportHeight;
      self.documentHeight = getContentHeight(item);
      if (self.documentHeight !== previousDocumentHeight) {
        calculateViewportI = self.watchers.length;
        while (calculateViewportI) {
          calculateViewportI -= 1;
          self.watchers[calculateViewportI].recalculateLocation();
        }

        previousDocumentHeight = self.documentHeight;
      }
    };

    var updateAndTriggerWatchersI = void 0;

    var updateAndTriggerWatchersInConstructor = function updateAndTriggerWatchersInConstructor() {
      // update all watchers then trigger the events so one can rely on another being up to date.
      updateAndTriggerWatchersI = self.watchers.length;
      while (updateAndTriggerWatchersI) {
        updateAndTriggerWatchersI -= 1;
        self.watchers[updateAndTriggerWatchersI].update();
      }

      updateAndTriggerWatchersI = self.watchers.length;
      while (updateAndTriggerWatchersI) {
        updateAndTriggerWatchersI -= 1;
        self.watchers[updateAndTriggerWatchersI].triggerCallbacks();
      }
    };

    this.createCustomContainer = function () {
      return new ScrollMonitorContainer();
    };

    this.update = function () {
      calculateViewport();
      updateAndTriggerWatchersInConstructor();
    };

    this.recalculateLocations = function () {
      _this.documentHeight = 0;
      _this.update();
    };

    this.destroy = function () {};
  } // constructor end

  _createClass(ScrollMonitorContainer, [{
    key: 'listenToDOM',
    value: function listenToDOM() {
      var _this2 = this;

      if (_constants.isInBrowser) {
        if (window.addEventListener) {
          if (this.item === document.body) {
            window.addEventListener('scroll', this.DOMListener);
          } else {
            this.item.addEventListener('scroll', this.DOMListener);
          }
          window.addEventListener('resize', this.DOMListener);
        } else {
          // Old IE support
          if (this.item === document.body) {
            window.attachEvent('onscroll', this.DOMListener);
          } else {
            this.item.attachEvent('onscroll', this.DOMListener);
          }
          window.attachEvent('onresize', this.DOMListener);
        }
        this.destroy = function () {
          if (window.addEventListener) {
            if (_this2.item === document.body) {
              window.removeEventListener('scroll', _this2.DOMListener);
              _this2.containerWatcher.destroy();
            } else {
              _this2.item.removeEventListener('scroll', _this2.DOMListener);
            }
            window.removeEventListener('resize', _this2.DOMListener);
          } else {
            // Old IE support
            if (_this2.item === document.body) {
              window.detachEvent('onscroll', _this2.DOMListener);
              _this2.containerWatcher.destroy();
            } else {
              _this2.item.detachEvent('onscroll', _this2.DOMListener);
            }
            window.detachEvent('onresize', _this2.DOMListener);
          }
        };
      }
    }
  }, {
    key: 'setStateFromDOM',
    value: function setStateFromDOM(event) {
      var viewportTop = scrollTop(this.item);
      var viewportHeight = getViewportHeight(this.item);
      var contentHeight = getContentHeight(this.item);

      this.setState(viewportTop, viewportHeight, contentHeight, event);
    }
  }, {
    key: 'setState',
    value: function setState(newViewportTop, newViewportHeight, newContentHeight, event) {
      var needsRecalcuate = newViewportHeight !== this.viewportHeight || newContentHeight !== this.contentHeight;
      this.latestEvent = event;
      this.viewportTop = newViewportTop;
      this.viewportHeight = newViewportHeight;
      this.viewportBottom = newViewportTop + newViewportHeight;
      this.contentHeight = newContentHeight;

      if (needsRecalcuate) {
        var i = this.watchers.length;
        while (i) {
          i -= 1;
          this.watchers[i].recalculateLocation();
        }
      }
      this.updateAndTriggerWatchers(event);
    }
  }, {
    key: 'updateAndTriggerWatchers',
    value: function updateAndTriggerWatchers(event) {
      var i = this.watchers.length;
      while (i) {
        i -= 1;
        this.watchers[i].update();
      }

      i = this.watchers.length;
      while (i) {
        i -= 1;
        this.watchers[i].triggerCallbacks(event);
      }
    }
  }, {
    key: 'createContainer',
    value: function createContainer(item) {
      if (typeof item === 'string') {
        item = document.querySelector(item);
      } else if (item && item.length > 0) {
        item = item[0];
      }
      var container = new ScrollMonitorContainer(item, this);
      container.setStateFromDOM();
      container.listenToDOM();
      return container;
    }
  }, {
    key: 'create',
    value: function create(item, offsets) {
      if (typeof item === 'string') {
        item = document.querySelector(item);
      } else if (item && item.length > 0) {
        item = item[0];
      }
      var watcher = new _watcher2.default(this, item, offsets);
      this.watchers.push(watcher);
      return watcher;
    }
  }, {
    key: 'beget',
    value: function beget(item, offsets) {
      return this.create(item, offsets);
    }
  }]);

  return ScrollMonitorContainer;
}();

module.exports = ScrollMonitorContainer;