'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _constants = require('./constants');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ElementWatcher = function () {
  function ElementWatcher(containerWatcher, watchItem, offsets) {
    var _this = this;

    _classCallCheck(this, ElementWatcher);

    var self = this;

    this.watchItem = watchItem;
    this.container = containerWatcher;

    if (!offsets) {
      this.offsets = _constants.defaultOffsets;
    } else if (offsets === +offsets) {
      this.offsets = { top: offsets, bottom: offsets };
    } else {
      this.offsets = {
        top: offsets.top || _constants.defaultOffsets.top,
        bottom: offsets.bottom || _constants.defaultOffsets.bottom
      };
    }

    var eventHandlerFactory = function eventHandlerFactory(type) {
      return function (callback, isOne) {
        _this.on(type, callback, isOne);
      };
    };

    for (var i = 0, j = _constants.eventTypes.length; i < j; i += 1) {
      this[_constants.eventTypes[i]] = eventHandlerFactory(_constants.eventTypes[i]);
    }

    this.callbacks = {}; // {callback: function, isOne: true }
    for (var _i = 0, _j = _constants.eventTypes.length; _i < _j; _i += 1) {
      this.callbacks[_constants.eventTypes[_i]] = [];
    }

    this.locked = false;

    var wasInViewport = void 0;
    var wasFullyInViewport = void 0;
    var wasAboveViewport = void 0;
    var wasBelowViewport = void 0;

    var listenerToTriggerListI = void 0;
    var listener = void 0;

    var triggerCallbackArray = function triggerCallbackArray(listeners, event) {
      if (listeners.length === 0) {
        return;
      }
      listenerToTriggerListI = listeners.length;
      while (listenerToTriggerListI) {
        listenerToTriggerListI -= 1;
        listener = listeners[listenerToTriggerListI];
        listener.callback.call(self, event, self);
        if (listener.isOne) {
          listeners.splice(listenerToTriggerListI, 1);
        }
      }
    };

    this.triggerCallbacks = function (event) {
      if (_this.isInViewport && !wasInViewport) {
        triggerCallbackArray(_this.callbacks[_constants.ENTERVIEWPORT], event);
      }
      if (_this.isFullyInViewport && !wasFullyInViewport) {
        triggerCallbackArray(_this.callbacks[_constants.FULLYENTERVIEWPORT], event);
      }

      if (_this.isAboveViewport !== wasAboveViewport && _this.isBelowViewport !== wasBelowViewport) {
        triggerCallbackArray(_this.callbacks[_constants.VISIBILITYCHANGE], event);
        // if you skip completely past this element
        if (!wasFullyInViewport && !_this.isFullyInViewport) {
          triggerCallbackArray(_this.callbacks[_constants.FULLYENTERVIEWPORT], event);
          triggerCallbackArray(_this.callbacks[_constants.PARTIALLYEXITVIEWPORT], event);
        }

        if (!wasInViewport && !_this.isInViewport) {
          triggerCallbackArray(_this.callbacks[_constants.ENTERVIEWPORT], event);
          triggerCallbackArray(_this.callbacks[_constants.EXITVIEWPORT], event);
        }
      }

      if (!_this.isFullyInViewport && wasFullyInViewport) {
        triggerCallbackArray(_this.callbacks[_constants.PARTIALLYEXITVIEWPORT], event);
      }

      if (!_this.isInViewport && wasInViewport) {
        triggerCallbackArray(_this.callbacks[_constants.EXITVIEWPORT], event);
      }

      if (_this.isInViewport !== wasInViewport) {
        triggerCallbackArray(_this.callbacks[_constants.VISIBILITYCHANGE], event);
      }

      switch (true) {
        case wasInViewport !== _this.isInViewport:
        case wasFullyInViewport !== _this.isFullyInViewport:
        case wasAboveViewport !== _this.isAboveViewport:
        case wasBelowViewport !== _this.isBelowViewport:
          triggerCallbackArray(_this.callbacks[_constants.STATECHANGE], event);
          break;

        default:
          return;
      }

      wasInViewport = _this.isInViewport;
      wasFullyInViewport = _this.isFullyInViewport;
      wasAboveViewport = _this.isAboveViewport;
      wasBelowViewport = _this.isBelowViewport;
    }; // triggerCallbacks end

    var recalculateLocation = function recalculateLocation() {
      if (_this.locked) {
        return;
      }
      var previousTop = _this.top;
      var previousBottom = _this.bottom;
      if (_this.watchItem.nodeName) {
        // a dom element
        var cachedDisplay = _this.watchItem.style.display;
        if (cachedDisplay === 'none') {
          _this.watchItem.style.display = '';
        }

        var containerOffset = 0;
        var container = _this.container;
        while (container.containerWatcher) {
          containerOffset += container.containerWatcher.top - container.containerWatcher.container.viewportTop;
          container = container.containerWatcher.container;
        }

        var boundingRect = _this.watchItem.getBoundingClientRect();
        _this.top = boundingRect.top + _this.container.viewportTop - containerOffset;
        _this.bottom = boundingRect.bottom + _this.container.viewportTop - containerOffset;

        if (cachedDisplay === 'none') {
          _this.watchItem.style.display = cachedDisplay;
        }
      } else if (_this.watchItem === +_this.watchItem) {
        // number
        if (_this.watchItem > 0) {
          _this.top = _this.bottom = _this.watchItem;
        } else {
          _this.top = _this.bottom = _this.container.documentHeight - _this.watchItem;
        }
      } else {
        // an object with a top and bottom property
        _this.top = _this.watchItem.top;
        _this.bottom = _this.watchItem.bottom;
      }

      _this.top -= _this.offsets.top;
      _this.bottom += _this.offsets.bottom;
      _this.height = _this.bottom - _this.top;

      if ((previousTop !== undefined || previousBottom !== undefined) && (_this.top !== previousTop || _this.bottom !== previousBottom)) {
        triggerCallbackArray(_this.callbacks[_constants.LOCATIONCHANGE], null);
      }
    }; // recalculateLocation end

    this.recalculateLocation = recalculateLocation;
    // this.triggerCallbacks = triggerCallbacks;

    recalculateLocation();
    this.update();

    wasInViewport = this.isInViewport;
    wasFullyInViewport = this.isFullyInViewport;
    wasAboveViewport = this.isAboveViewport;
    wasBelowViewport = this.isBelowViewport;

    this.isAboveViewportArr = [];
  } // constructor end

  _createClass(ElementWatcher, [{
    key: 'on',
    value: function on(event, callback, isOne) {
      // trigger the event if it applies to the element right now.
      switch (true) {
        case event === _constants.VISIBILITYCHANGE && !this.isInViewport && this.isAboveViewport:
        case event === _constants.ENTERVIEWPORT && this.isInViewport:
        case event === _constants.FULLYENTERVIEWPORT && this.isFullyInViewport:
        case event === _constants.EXITVIEWPORT && this.isAboveViewport && !this.isInViewport:
        case event === _constants.PARTIALLYEXITVIEWPORT && this.isAboveViewport:
        default:
          callback.call(this, this.container.latestEvent, this);
          if (isOne) {
            return;
          }
          break;
      }

      if (this.callbacks[event]) {
        this.callbacks[event].push({ callback: callback, isOne: isOne || false });
      } else {
        throw new Error('Tried to add a scroll monitor listener of type-' + event + '-Your options are:-' + _constants.eventTypes.join(', '));
      }
    }
  }, {
    key: 'off',
    value: function off(event, callback) {
      if (this.callbacks[event]) {
        // for has issue  item = this.callbacks[event][i]
        for (var i = 0, item; item === this.callbacks[event][i]; i += 1) {
          if (item.callback === callback) {
            this.callbacks[event].splice(i, 1);
            break;
          }
        }
      } else {
        throw new Error('Tried to remove a scroll monitor listener of type-' + event + '-Your options are:-' + _constants.eventTypes.join(', '));
      }
    }
  }, {
    key: 'one',
    value: function one(event, callback) {
      this.on(event, callback, true);
    }
  }, {
    key: 'recalculateSize',
    value: function recalculateSize() {
      this.height = this.watchItem.offsetHeight + this.offsets.top + this.offsets.bottom;
      this.bottom = this.top + this.height;
    }
  }, {
    key: 'update',
    value: function update() {
      this.isAboveViewport = this.top < this.container.viewportTop;
      this.isBelowViewport = this.bottom > this.container.viewportBottom;

      this.isInViewport = this.top < this.container.viewportBottom && this.bottom > this.container.viewportTop;
      this.isFullyInViewport = this.top >= this.container.viewportTop && (this.bottom <= this.container.viewportBottom || this.isAboveViewport && this.isBelowViewport);
    }
  }, {
    key: 'destroy',
    value: function destroy() {
      var index = this.container.watchers.indexOf(this);
      var self = this;
      this.container.watchers.splice(index, 1);
      for (var i = 0, j = _constants.eventTypes.length; i < j; i += 1) {
        self.callbacks[_constants.eventTypes[i]].length = 0;
      }
    }
  }, {
    key: 'lock',
    value: function lock() {
      this.locked = true;
    }
  }, {
    key: 'unlock',
    value: function unlock() {
      this.locked = false;
    }
  }]);

  return ElementWatcher;
}();

module.exports = ElementWatcher;